import { CancellationToken } from 'vscode-jsonrpc';
import { CompletionList, DidChangeTextDocumentParams, DidCloseTextDocumentParams, DidOpenTextDocumentParams, DidSaveTextDocumentParams, DocumentSymbolParams, Hover, InitializeResult, Location, ReferenceParams, SymbolInformation, TextDocumentPositionParams } from 'vscode-languageserver';
import { DependencyReference, InitializeParams, PackageInformation, ReferenceInformation, SymbolLocationInformation, WorkspaceReferenceParams, WorkspaceSymbolParams } from './request-type';
import { FileSystem } from './fs';
import * as pm from './project-manager';
import { LanguageClientHandler, LanguageHandler } from './lang-handler';
export interface TypeScriptServiceOptions {
    traceModuleResolution?: boolean;
    strict?: boolean;
}
/**
 * TypeScriptService handles incoming requests and return
 * responses. There is a one-to-one-to-one correspondence between TCP
 * connection, TypeScriptService instance, and language
 * workspace. TypeScriptService caches data from the compiler across
 * requests. The lifetime of the TypeScriptService instance is tied to
 * the lifetime of the TCP connection, so its caches are deleted after
 * the connection is torn down.
 */
export declare class TypeScriptService implements LanguageHandler {
    protected client: LanguageClientHandler;
    protected options: TypeScriptServiceOptions;
    projectManager: pm.ProjectManager;
    root: string;
    private emptyQueryWorkspaceSymbols;
    private traceModuleResolution;
    protected fileSystem: FileSystem;
    constructor(client: LanguageClientHandler, options?: TypeScriptServiceOptions);
    initialize(params: InitializeParams, token?: CancellationToken): Promise<InitializeResult>;
    /**
     * Called before the ProjectManager is initialized, after the FileSystem was initialized
     */
    protected beforeProjectInit(): Promise<void>;
    shutdown(): Promise<void>;
    getDefinition(params: TextDocumentPositionParams, token?: CancellationToken): Promise<Location[]>;
    getXdefinition(params: TextDocumentPositionParams, token?: CancellationToken): Promise<SymbolLocationInformation[]>;
    getHover(params: TextDocumentPositionParams, token?: CancellationToken): Promise<Hover>;
    getReferences(params: ReferenceParams, token?: CancellationToken): Promise<Location[]>;
    getWorkspaceSymbols(params: WorkspaceSymbolParams, token?: CancellationToken): Promise<SymbolInformation[]>;
    getWorkspaceSymbolsDefinitelyTyped(params: WorkspaceSymbolParams, token?: CancellationToken): Promise<SymbolInformation[] | null>;
    getDocumentSymbol(params: DocumentSymbolParams, token?: CancellationToken): Promise<SymbolInformation[]>;
    getWorkspaceReference(params: WorkspaceReferenceParams, token?: CancellationToken): Promise<ReferenceInformation[]>;
    getPackages(params?: {}, token?: CancellationToken): Promise<PackageInformation[]>;
    getDependencies(params?: {}, token?: CancellationToken): Promise<DependencyReference[]>;
    getCompletions(params: TextDocumentPositionParams, token?: CancellationToken): Promise<CompletionList>;
    private walkMostAST(node, visit);
    didOpen(params: DidOpenTextDocumentParams): Promise<void>;
    didChange(params: DidChangeTextDocumentParams): Promise<void>;
    didSave(params: DidSaveTextDocumentParams): Promise<void>;
    didClose(params: DidCloseTextDocumentParams): Promise<void>;
    /**
     * Fetches (or creates if needed) source file object for a given file name
     * @param configuration project configuration
     * @param fileName file name to fetch source file for or create it
     */
    private getSourceFile(configuration, fileName);
    /**
     * Produces async function that converts ReferenceEntry object to Location
     */
    private transformReference(root, program, includeDeclaration, ref);
    /**
     * transformNavItem transforms a NavigateToItem instance to a SymbolInformation instance
     */
    private transformNavItem(root, program, item);
    private collectWorkspaceSymbols(configs, query?, symQuery?);
    /**
     * Transforms definition's file name to URI. If definition belongs to TypeScript library,
     * returns git://github.com/Microsoft/TypeScript URL, otherwise returns file:// one
     */
    private defUri(filePath);
    /**
     * Fetches up to limit navigation bar items from given project, flattens them
     */
    private getNavigationTreeItems(configuration, limit?);
    /**
     * Flattens navigation tree by transforming it to one-dimensional array.
     * Some items (source files, modules) may be excluded
     */
    private flattenNavigationTreeItem(item, parent, sourceFile, result, limit?);
    /**
     * Transforms NavigationTree to SymbolInformation
     */
    private transformNavigationTreeItem(item, parent, sourceFile);
    /**
     * @return true if navigation tree item is acceptable for inclusion into workspace/symbols
     */
    private static isAcceptableNavigationTreeItem(item);
}
